/***************************************************************************
 *   Copyright (C) 2008 by Alexey Noskov   *
 *   alexey.noskov@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kopeteantispamplugin.h"
#include "kopeteantispamconfig.h"

#include <kopete/kopetechatsession.h>
#include <kopete/kopetechatsessionmanager.h>
#include <kopete/kopeteuiglobal.h>
#include <kopete/kopetecontact.h>
#include <kopete/kopetemessage.h>
#include <kopete/kopetemetacontact.h>
#include <kopete/kopetecontactlist.h>
#include <kopete/kopetegroup.h>

#include <kopete/ui/kopeteview.h>

#include <kgenericfactory.h>

#include <qfileinfo.h>
#include <qpixmap.h>

#include <iostream>

KopeteAntispamPlugin *KopeteAntispamPlugin::instance = 0;

K_PLUGIN_FACTORY( KopeteAntispamPluginFactory, registerPlugin<KopeteAntispamPlugin>(); )
K_EXPORT_PLUGIN( KopeteAntispamPluginFactory ( "kopete_antispam" ) )

KopeteAntispamPlugin::KopeteAntispamPlugin( QObject *parent, const QVariantList & /*args*/ ) :
	Kopete::Plugin( KopeteAntispamPluginFactory::componentData(), parent ), question( "" ), answer( "" )
{
	if ( instance != 0 ) // Нельзя создать второй экземпляр
		return;

	instance = this; // Сохраняем ссылку на экземпляр плагина

	QTimer::singleShot( 1000, this, SLOT( init() ) ); // Инциализируем по таймеру
}

void KopeteAntispamPlugin::init()
{
	connect( Kopete::ChatSessionManager::self() , SIGNAL( aboutToReceive( Kopete::Message & ) ) , this, SLOT( onMessageReceive( Kopete::Message & ) ) );

	connect( this , SIGNAL( settingsChanged() ) , this , SLOT( settingsChanged() ) );

	settingsChanged();
}

void KopeteAntispamPlugin::settingsChanged()
{
	KopeteAntispamConfig * config = KopeteAntispamConfig::self();

	config->readConfig();

	question = config->question();
	answer = config->answer();
	passMessage = config->passMessage();

	ignoreUnknown = config->ignoreUnknown();
	removeSpammers = config->removeSpammers();

	updateWhiteList( config->whiteList() );
}

void KopeteAntispamPlugin::updateWhiteList( const QString & str )
{
	QStringList tokens = str.split( '\n' );

	whiteList.clear();

	for ( QStringList::iterator it = tokens.begin(), end = tokens.end(); it != end; ++ it )
		whiteList.push_back( QRegExp( it->trimmed(), Qt::CaseInsensitive, QRegExp::Wildcard ) ); // Found a token, add regexp it to the vector
}

bool KopeteAntispamPlugin::contactInWhiteList( const Kopete::Contact * contact )
{
	if ( !contact ) return false;

	const QString & str = contact->contactId();

	for ( WhiteList::iterator it = whiteList.begin(); it != whiteList.end(); ++ it ) // For each pattern in white list
		if ( it->exactMatch( str ) ) // If pattern matches contact
			return true;

	return false;
}

void KopeteAntispamPlugin::removeContact( Kopete::Contact * contact ) {
	Kopete::MetaContact* metacontact = contact->metaContact();

	// Remove the Contact
	metacontact->removeContact( contact );
	contact->deleteLater();

	// No remaining contacts for metacontact, so remove the MetaContact
	if ( metacontact->contacts().count() == 0 ) {
		// Get the first group and its members
		Kopete::Group* group = metacontact->groups().first();
		QList<Kopete::MetaContact *> groupMembers = group->members();
		Kopete::ContactList::self()->removeMetaContact( metacontact );

		// Remove the group
		if ( groupMembers.count() == 1 && groupMembers.indexOf( metacontact ) != -1 ) {
			Kopete::ContactList::self()->removeGroup( group );
		}
	}
}

void KopeteAntispamPlugin::removeSessionContacts() {
	const Kopete::ChatSession * session = dynamic_cast<const Kopete::ChatSession *>( sender() );
	const Kopete::ContactPtrList & list = session->members();

	for ( Kopete::ContactPtrList::ConstIterator it = list.begin(); it != list.end(); ++ it ) {
		if ( *it != session->myself() ) {
			removeContact( *it );
		}
	}
}

void KopeteAntispamPlugin::onMessageReceive( Kopete::Message & msg )
{
	Kopete::ChatSession & session = *msg.manager(); // Get session, in which we receive message

	if ( ( !session.view() ) && // If no view assotiated with session
	     ( !msg.from() || !msg.from()->metaContact() || msg.from()->metaContact()->isTemporary() ) && // And contact isn't in contact list
	     ( !contactInWhiteList( msg.from() ) ) ) { // And contact not in white list

		if ( msg.plainBody() == answer && !ignoreUnknown ) { // Message is answer
			Kopete::Message msgPass( session.myself(), msg.from() ); // We create message with notification about passing test
			msgPass.setDirection( Kopete::Message::Outbound );
			msgPass.setHtmlBody( passMessage );

			session.sendMessage( msgPass ); // Send message
		} else { // Message isn't answer
			if ( !ignoreUnknown ) { // If ignore mode not set, send question to contact
				Kopete::Message msgQuestion( session.myself(), msg.from() ); // We create message with question
				msgQuestion.setDirection( Kopete::Message::Outbound );
				msgQuestion.setHtmlBody( question );

				if ( removeSpammers ) {
					connect( &session, SIGNAL( messageSuccess() ), this, SLOT( removeSessionContacts() ) );
				}

				session.sendMessage( msgQuestion ); // Send message
			} else {
				if ( removeSpammers ) { // If remove spammers flag is set, remove contact from list
					removeContact( (Kopete::Contact*) msg.from() );
				}
			}

			disconnect( &session, SIGNAL( messageAppended( Kopete::Message &, Kopete::ChatSession * ) ), 0, 0 ); // Disconnect session
		}
	}
}
